//
// Authors:
//   Patrick Cernko <errror@ps.uni-sb.de>
//
// Copyright:
//   Patrick Cernko, 2004
//
// Last Change:
//   $Date: 2004/01/21 11:02:27 $ by $Author: errror $
//   $Revision: 1.4 $
//

#if defined(INTERFACE)
#pragma implementation "AEData.hh"
#endif

#include "AEData.hh"


////////////////////////////////////////////////////////////////////////////////
// AENode
////////////////////////////////////////////////////////////////////////////////
AENode* AENode::New(ArithmeticOperation operation,
                    word left, word right) {
  // alloc a block
  Block* b = Store::AllocBlock(MIN_DATA_LABEL, SIZE);

  // and store the given arguments at their positions
  b->InitArg(OPERATION_POS,  Store::IntToWord(operation));
  b->InitArg(LEFT_NODE_POS,  left);
  b->InitArg(RIGHT_NODE_POS, right);

  // return it proper casted
  return STATIC_CAST(AENode *, b);
}

AENode* AENode::New(ArithmeticOperation operation,
                    AENode* left, s_int right) {
  return New(operation,
             left->ToWord(),
             Store::IntToWord(right));
}

AENode* AENode::New(ArithmeticOperation operation,
                    AENode* left, AENode* right) {
  return New(operation,
             left->ToWord(),
             right->ToWord());
}

AENode* AENode::New(ArithmeticOperation operation,
                    s_int left, AENode* right) {
  return New(operation,
             Store::IntToWord(left),
             right->ToWord());
}

AENode* AENode::New(ArithmeticOperation operation,
                    s_int left, s_int right) {
  return New(operation,
             Store::IntToWord(left),
             Store::IntToWord(right));
}

AENode* AENode::FromWord(word x) {
  // make word to a block
  Block* b = Store::WordToBlock(x);
  if (b == INVALID_POINTER) {
    // this is not a AENode but a s_int
    return INVALID_POINTER;
  } else {
    // assert the block is of correct type
    Assert(b->GetLabel() == AE::AENode);
    // return it proper casted
    return STATIC_CAST(AENode *, b);
  }
}

word AENode::GetLeft() {
  // get our argument at LEFT_NODE_POS
  // and return it correctly typed as a pointer to a AENode
  return GetArg(LEFT_NODE_POS);
}

word AENode::GetRight() {
  // get our argument at RIGHT_NODE_POS
  // and return it correctly typed as a pointer to a AENode
  return GetArg(RIGHT_NODE_POS);
}

AENode* AENode::GetLeftExpression() {
  // get our argument at LEFT_NODE_POS
  // and return it correctly typed as a pointer to a AENode
  return AENode::FromWord(GetArg(LEFT_NODE_POS));
}

AENode* AENode::GetRightExpression() {
  // get our argument at RIGHT_NODE_POS
  // and return it correctly typed as a pointer to a AENode
  return AENode::FromWord(GetArg(RIGHT_NODE_POS));
}

s_int AENode::GetLeftNumber() {
  // get our argument at LEFT_NODE_POS
  // and return it correctly typed as s_int
  return Store::WordToInt(GetArg(LEFT_NODE_POS));
}

s_int AENode::GetRightNumber() {
  // get our argument at RIGHT_NODE_POS
  // and return it correctly typed as a pointer to a AENode
  return Store::WordToInt(GetArg(RIGHT_NODE_POS));
}

AENode::ArithmeticOperation AENode::GetArithmeticOperation() {
  // get our argument at OPERATION_POS
  // and return it correctly typed as ArithmeticOperation
  return STATIC_CAST(ArithmeticOperation, Store::WordToInt(GetArg(OPERATION_POS)));
}

void AENode::Dump(u_int indent) {
  // dump operation
  switch (GetArithmeticOperation()) {
  case PLUS:
    fprintf(stderr, "PLUS\n");
    break;
  case MINUS:
    fprintf(stderr, "MINUS\n");
    break;
  case MUL:
    fprintf(stderr, "MUL\n");
    break;
  case DIV:
    fprintf(stderr, "DIV\n");
    break;
  case INVALID_OPERATION:
    fprintf(stderr, "INVALID_OPERATION\n");
    return;
    break;
  }

  // indent
  for (u_int i = 0; i < indent; i++) { fprintf(stderr, " "); }
  // create subtree symbol
  fprintf(stderr, " \\");
  // dump left recursively
  Dump(GetLeft(), indent+2);

  // indent
  for (u_int i = 0; i < indent; i++) { fprintf(stderr, " "); }
  // create subtree symbol
  fprintf(stderr, " \\");
  // dump right recursively
  Dump(GetRight(), indent+2);
}

void AENode::Dump(word node, u_int indent) {
  AENode* tree = FromWord(node);
  if (tree != INVALID_POINTER) {
    tree->Dump(indent);
  } else {
    fprintf(stderr, "%d\n", Store::WordToInt(node));
  }
}
////////////////////////////////////////////////////////////////////////////////



