//
// Authors:
//   Patrick Cernko <errror@ps.uni-sb.de>
//
// Copyright:
//   Patrick Cernko, 2004
//
// Last Change:
//   $Date: 2004/01/21 11:02:27 $ by $Author: errror $
//   $Revision: 1.5 $
//

#ifndef __AE__AE_DATA_HH__
#define __AE__AE_DATA_HH__

#if defined(INTERFACE)
#pragma interface "AEData.hh"
#endif

#include "Base.hh"

/**
 * \class AENode
 *
 * A aritmetic expression node for a parse tree.
 */
class AENode
  : private Block {

private:
  // member enum
  enum { 
    OPERATION_POS,  ///! the position of the stored aritmetic
                    ///  operation (+,-,*,/)
    LEFT_NODE_POS,  ///! the position of the stored left aritmetic
                    ///  expression subtree
    RIGHT_NODE_POS, ///! the position of the stored left aritmetic
                    ///  expression subtree
    SIZE            ///! last position, always the size
  };

public:
  enum ArithmeticOperation {
    INVALID_OPERATION,
    PLUS,  ///! indicates, that the result of the left and right
           ///  subexpression should be added
    MINUS, ///! indicates, that the result of the right subexpression
           ///  should be substracted from the result of the left one
    MUL,   ///! indicates, that the result of the left and right
           ///  subexpressions should be multiplicated
    DIV,   ///! indicates, that the result of the left subexpression
           ///  should be divided by the result of the left one
  };

  using Block::ToWord;

  /**
   * a "Constructor":
   *
   * \param operation The operation between the two subtrees.
   * \param left      The left subtree.
   * \param right     The right subtree.
   */
  static AENode* New(ArithmeticOperation operation,
                     word left, word right);

  /**
   * a "Constructor":
   *
   * \param operation The operation between the two subtrees.
   * \param left      The left subtree as Node.
   * \param right     The right subtree as number.
   */
  static AENode* New(ArithmeticOperation operation,
                     AENode* left, s_int right);

  /**
   * a "Constructor":
   *
   * \param operation The operation between the two subtrees.
   * \param left      The left subtree as Node.
   * \param right     The right subtree as Node.
   */
  static AENode* New(ArithmeticOperation operation,
                     AENode* left, AENode* right);

  /**
   * a "Constructor":
   *
   * \param operation The operation between the two subtrees.
   * \param left      The left subtree as number.
   * \param right     The right subtree as Node.
   */
  static AENode* New(ArithmeticOperation operation,
                     s_int left, AENode* right);

  /**
   * a "Constructor":
   *
   * \param operation The operation between the two subtrees.
   * \param left      The left subtree as number.
   * \param right     The right subtree as number.
   */
  static AENode* New(ArithmeticOperation operation,
                     s_int left, s_int right);

  /**
   * another "Constructor":
   * This one is for casts from Store-words
   * \param x The word to be reinterpreted as AENode or
   *          INVALID_POINTER if word does not represent a AENode.
   */
  static AENode* FromWord(word x);

  /**
   * the left node.
   * \returns the left node as word.
   */
  word GetLeft();

  /**
   * the right node.
   * \returns the right node as word.
   */
  word GetRight();

  /**
   * the left expression term.
   * \returns a crrectly typed pointer to the stored arithmetic
   *          expression on the left or INVALID_POINTER if the
   *          expression is a number.
   */
  AENode* GetLeftExpression();

  /**
   * the right expression term.
   * \returns a crrectly typed pointer to the stored arithmetic
   *          expression on the right or INVALID_POINTER if the
   *          expression is a number.
   */
  AENode* GetRightExpression();

  /**
   * the left expression term.
   * \returns a crrectly the stored number on the left or INVALID_INT
   *          if the node on the left is not a number.
   */
  s_int GetLeftNumber();

  /**
   * the left expression term.
   * \returns a crrectly the stored number on the right or INVALID_INT
   *          if the node on the right is not a number.
   */
  s_int GetRightNumber();

  /**
   * the stored arithmetic operation.
   * \returns the type of the operation between the two subexpressions.
   */
  ArithmeticOperation GetArithmeticOperation();

  void Dump(u_int indent);

  static void Dump(word node, u_int indent);
};

#endif
