//
// Authors:
//   Patrick Cernko <errror@ps.uni-sb.de>
//
// Copyright:
//   Patrick Cernko, 2004
//
// Last Change:
//   $Date: 2004/01/21 15:25:29 $ by $Author: errror $
//   $Revision: 1.2 $
//

#if defined(INTERFACE)
#pragma implementation "AEEvalFrame.hh"
#endif

#include "AEEvalFrame.hh"
#include "AEData.hh"

////////////////////////////////////////////////////////////////////////////////
// AEEvalFrame
////////////////////////////////////////////////////////////////////////////////
AEEvalFrame* AEEvalFrame::New(Thread* thread,
                              Interpreter* interpreter) {
  // create a frame on the given thread, using the given interpreter
  // reas as: StackFrame* frame = thread->CreateStackFrameUsingInterpreter(interpreter, SIZE);
  NEW_THREAD_STACK_FRAME(frame, thread, interpreter, SIZE);
  // comment: SIZE is the LAST enum in our member enum, so we have
  // ((int) SIZE) members!

  // set the first argument for the created frame
  frame->InitArg(NODE_POS, 0);

  // return correctly typed frame
  return STATIC_CAST(AEEvalFrame *, frame);
}

AEEvalFrame* AEEvalFrame::New(Interpreter* interpreter) {
  // create a frame, using the given interpreter
  // reas as:
  // StackFrame* frame = cur_thread->CreateStackFrameUsingInterpreter(interpreter, SIZE);
  NEW_STACK_FRAME(frame, interpreter, SIZE);
  // comment: SIZE is the LAST enum in our member enum, so we have
  // ((int) SIZE) members!

  // set the first argument for the created frame
  frame->InitArg(NODE_POS, 0);

  // return correctly typed frame
  return STATIC_CAST(AEEvalFrame *, frame);
}

u_int AEEvalFrame::GetSize() {
  // the size of this frame is the size of the inherited StackFrame
  // plus the number of members we have, provided as last enum in the
  // members enum!
  return StackFrame::GetSize() + SIZE;
}
 
AENode* AEEvalFrame::GetNode() {
  // get our argument at NODE_POS
  // and return it as a AENode
  return AENode::FromWord(GetArg(NODE_POS));
}

void AEEvalFrame::SetNode(AENode* node) {
  ReplaceArg(NODE_POS, node->ToWord());
}
////////////////////////////////////////////////////////////////////////////////
