//
// Authors:
//   Patrick Cernko <errror@ps.uni-sb.de>
//
// Copyright:
//   Patrick Cernko, 2004
//
// Last Change:
//   $Date: 2004/01/21 15:24:39 $ by $Author: errror $
//   $Revision: 1.3 $
//

#ifndef __AE__AE_EVAL_FRAME_HH__
#define __AE__AE_EVAL_FRAME_HH__

#if defined(INTERFACE)
#pragma interface "AEEvalFrame.hh"
#endif

#include "Base.hh"

class AENode;

/**
 * \class AEEvalFrame
 *
 * A StackFrame for the AEEvalInterpreter.
 */
class AEEvalFrame
  : private StackFrame {

private:
  // member enum
  enum { 
    NODE_POS,      ///! the position of the stored aritmetic expression (sub-)string
    SIZE           ///! last position, always the size
  };

public:
  /**
   * a "Constructor":
   * this one is meant to be called only on startup. It is created on
   * the given thread with the given interpreter and the arithmetic
   * expression to be stored in it.
   *
   * \param thread      The thread on which the frame should be created.
   * \param interpreter The interpreter for this frame.
   * \param tree        The arithmetic expression abstract syntax tree.
   */
  static AEEvalFrame* New(Thread* thread, Interpreter* interpreter);

  /**
   * a "Constructor":
   * this is th "normal" one. It is created with the given interpreter
   * and the arithmetic expression to be stored in it.
   *
   * \param interpreter The interpreter for this frame.
   * \param tree        The arithmetic expression.
   */
  static AEEvalFrame* New(Interpreter* interpreter);

  /**
   * the size of this frame.
   * \returns the size of this frame.
   */
  u_int GetSize();

  /**
   * the stored arithmetic expression node.
   * \returns the to be parsed parsed node.
   */
  AENode* GetNode();

  /**
   * stores the to be parsed parsed Node.
   */
  void SetNode(AENode* node);
};

#endif
