//
// Authors:
//   Patrick Cernko <errror@ps.uni-sb.de>
//
// Copyright:
//   Patrick Cernko, 2004
//
// Last Change:
//   $Date: 2004/01/21 09:53:09 $ by $Author: errror $
//   $Revision: 1.3 $
//

#if defined(INTERFACE)
#pragma implementation "AEFrame.hh"
#endif

#include "AEFrame.hh"
#include "AEData.hh"

////////////////////////////////////////////////////////////////////////////////
// AE Frame
////////////////////////////////////////////////////////////////////////////////
AEFrame* AEFrame::New(Thread* thread,
                      Interpreter* interpreter,
                      String* ae) {
  // create a frame on the given thread, using the given interpreter
  // reas as: StackFrame* frame = thread->CreateStackFrameUsingInterpreter(interpreter, SIZE);
  NEW_THREAD_STACK_FRAME(frame, thread, interpreter, SIZE);
  // comment: SIZE is the LAST enum in our member enum, so we have
  // ((int) SIZE) members!

  // set the given ae as first argument for the created frame
  frame->InitArg(AE_POS, ae->ToWord());
  // set the second argument for the created frame
  frame->InitArg(PARSE_EX_POS, Store::IntToWord(PARSE_EXPRESSION));
  // set the rest of the arguments to invalid
  frame->InitArg(NODE_POS, 0); // is it ok to store a (int) 0 as
                               // invalid entry and replace it with a
                               // block ptr later?
  // set the second argument for the created frame
  frame->InitArg(OP_POS, Store::IntToWord(AENode::INVALID_OPERATION));

  // return correctly typed frame
  return STATIC_CAST(AEFrame *, frame);
}

AEFrame* AEFrame::New(Interpreter* interpreter,
                      String* ae,
                      ParsingAction pe) {
  // create a frame, using the given interpreter
  // reas as:
  // StackFrame* frame = cur_thread->CreateStackFrameUsingInterpreter(interpreter, SIZE);
  NEW_STACK_FRAME(frame, interpreter, SIZE);
  // comment: SIZE is the LAST enum in our member enum, so we have
  // ((int) SIZE) members!

  // set the given ae as first argument for the created frame
  frame->InitArg(AE_POS, ae->ToWord());
  // set the second argument for the created frame to the given ParsingAction
  frame->InitArg(PARSE_EX_POS, Store::IntToWord(pe));
  // set the rest of the arguments to invalid
  frame->InitArg(NODE_POS, 0); // is it ok to store a (int) 0 as
                               // invalid entry and replace it with a
                               // block ptr later?
  // set the second argument for the created frame
  frame->InitArg(OP_POS, Store::IntToWord(AENode::INVALID_OPERATION));

  // return correctly typed frame
  return STATIC_CAST(AEFrame *, frame);
}

u_int AEFrame::GetSize() {
  // the size of this frame is the size of the inherited StackFrame
  // plus the number of members we have, provided as last enum in the
  // members enum!
  return StackFrame::GetSize() + SIZE;
}
 
String* AEFrame::GetAE() {
  // get our argument at AE_POS
  // and return a pointer correctly typed as String from it
  return String::FromWordDirect(GetArg(AE_POS));
}

AEFrame::ParsingAction AEFrame::GetParsingAction() {
  // get our argument at PARSE_EX_POS
  // and return it correctly typed as ParsingAction
  return STATIC_CAST(ParsingAction, Store::WordToInt(GetArg(PARSE_EX_POS)));
}

word AEFrame::GetNode() {
  // get our argument at NODE_POS
  // and return it as a word
  return GetArg(NODE_POS);
}

void AEFrame::SetNode(word node) {
  ReplaceArg(NODE_POS, node);
}

s_int AEFrame::GetOperation() {
  // get our argument at OP_POS
  // and return it as a s_int
  return Store::WordToInt(GetArg(OP_POS));
}

void AEFrame::SetOperation(s_int operation) {
  // get our argument at OP_POS
  // and return it as a s_int
  ReplaceArg(OP_POS, Store::IntToWord(operation));
}
////////////////////////////////////////////////////////////////////////////////
