//
// Authors:
//   Patrick Cernko <errror@ps.uni-sb.de>
//
// Copyright:
//   Patrick Cernko, 2004
//
// Last Change:
//   $Date: 2004/01/21 14:56:46 $ by $Author: errror $
//   $Revision: 1.4 $
//

#ifndef __AE__AE_FRAME_HH__
#define __AE__AE_FRAME_HH__

#if defined(INTERFACE)
#pragma interface "AEFrame.hh"
#endif

#include "Base.hh"

/**
 * \class AEFrame
 *
 * A StackFrame for the AEInterpreter, containing the arithmetic
 * expression to be parsed.
 */
class AEFrame
  : private StackFrame {

private:
  // member enum
  enum { 
    AE_POS,       ///! the position of the stored aritmetic expression (sub-)string
    PARSE_EX_POS, ///! the position of the stored next parsing action to make
    NODE_POS,     ///! the position of the stored node already parsed
    OP_POS,       ///! the position of the stored operator already parsed
    SIZE          ///! last position, always the size
  };

public:
  enum ParsingAction {
    PARSE_EXPRESSION, ///! indicates that the AEInterpreter should
                      ///  take the stored ae substring as a complete
                      ///  expression
    PARSE_TERM,       ///! indicates that the AEInterpreter should
                      ///  take the stored ae substring as a term
                      ///  expression
    PARSE_NUMBER,     ///! indicates that the AEInterpreter should
                      ///  take the stored ae substring as a number
  };

  /**
   * a "Constructor":
   * this one is meant to be called only on startup. It is created on
   * the given thread with the given interpreter and the arithmetic
   * expression to be stored in it.
   *
   * \param thread      The thread on which the frame should be created.
   * \param interpreter The interpreter for this frame.
   * \param ae          The arithmetic expression to be stored in the frame.
   */
  static AEFrame* New(Thread* thread, Interpreter* interpreter, String* ae);

  /**
   * a "Constructor":
   * this is th "normal" one. It is created with the given interpreter
   * and the arithmetic expression to be stored in it.
   *
   * \param interpreter The interpreter for this frame.
   * \param ae          The arithmetic expression to be stored in the frame.
   * \param pe          The next parsing action to be stored in the frame.
   */
  static AEFrame* New(Interpreter* interpreter, String* ae, ParsingAction pe);

  /**
   * the size of this frame.
   * \returns the size of this frame.
   */
  u_int GetSize();

  /**
   * the stored arithmetic expression (sub-)string.
   * \returns a crrectly typed pointer to the stored arithmetic
   *          expression (sub-)string.
   */
  String* GetAE();

  /**
   * the stored arithmetic expression.
   * \returns the type of the action for parsing the contained ae substring.
   */
  ParsingAction GetParsingAction();

  /**
   * the stored arithmetic expression node.
   * \returns a word representing the already parsed node.
   */
  word GetNode();

  /**
   * stores the already parsed Node as a word
   */
  void SetNode(word node);

  /**
   * the stored arithmetic operation already parsed.
   * \returns a u_int representing the already parsed arithmetic opereation.
   */
  s_int GetOperation();

  /**
   * stores the already parsed arithmetic operation as a i_unt
   */
  void SetOperation(s_int operation);

};

#endif
