//
// Author:
//   Patrick Cernko <errror@ps.uni-sb.de>
//
// Copyright:
//   Patrick Cernko, 2004
//
// Last Change:
//   $Date: 2004/01/21 17:51:22 $ by $Author: errror $
//   $Revision: 1.8 $
//
// Base on AliceLanguageLayer.cc from alice-on-seam
// Author:
//   Leif Kornstaedt <kornstae@ps.uni-sb.de>
//
// Copyright:
//   Leif Kornstaedt, 2002-2003
//

#if defined(INTERFACE)
#pragma implementation "AELanguageLayer.hh"
#endif

#include "AELanguageLayer.hh"

#include "AEInterpreter.hh"
#include "AEEvalInterpreter.hh"
#include "AEPrintInterpreter.hh"

#include <getopt.h>

// initialize the static members
word AELanguageLayer::cmdAE;
u_int AELanguageLayer::verbosity = 0;

void usage(const char* name) {
  // print usage if wrong count!
  // todo? : better soluting than using argv[0] == "/home/errror/.../ae"
  std::fprintf(stderr, "usage: %s \"<arithmetic expression>\"\n", name);
}

void AELanguageLayer::Init(int argc, const char* argv[]) {
  // get options with getopt
  {
    const char* optstring = "vh";
    struct option longopts[] = {
      {"verbose", no_argument, NULL, 'v'},
      {"help",    no_argument, NULL, 'h'},
      {NULL,      0,           NULL, 0  } /* last row must be zero */
    };
  
    /* some parsing variables */
    int ch, optionindex;
    while(EOF != (ch=getopt_long(argc,
                                 (char**) argv,
                                 optstring,
                                 longopts,
                                 &optionindex))){
      // which option did we get?
      switch (ch) {
      case 'v':
        verbosity++;
        break;

      case 'h':
        // fallthrough
      default:
        usage(argv[0]);
        std::exit(2);
      }
    }

    // get non-option argument, the file to parse
    if (optind+1 != argc) {
      usage(argv[0]);
      std::exit(2);
    }

    // store the first non-option argument (the arithmetic expression)
    cmdAE = String::New(argv[optind])->ToWord();
    // add the argument to the RootSet for GC
    RootSet::Add(cmdAE);
  }

  // Initialize our interpreter
  AEInterpreter::Init();
  // Initialize our evaluator
  AEEvalInterpreter::Init();
  // Initialize our printer
  AEPrintInterpreter::Init();
}

String* AELanguageLayer::GetCmdAE() {
  // return cmdAE correctly type-converted
  return String::FromWordDirect(cmdAE);
}
