//
// Authors:
//   Patrick Cernko <errror@ps.uni-sb.de>
//
// Copyright:
//   Patrick Cernko, 2004
//
// Last Change:
//   $Date: 2004/01/21 17:51:22 $ by $Author: errror $
//   $Revision: 1.9 $
//
// Based on AliceMain.cc from alice-on-seam
// Authors:
//   Thorsten Brunklaus <brunklaus@ps.uni-sb.de>
//   Leif Kornstaedt <kornstae@ps.uni-sb.de>
//
// Copyright:
//   Thorsten Brunklaus, 2002
//   Leif Kornstaedt, 2002-2003
//

#include <cstdio> // for exit(), fprintf()

#include "AELanguageLayer.hh"
#include "AEInterpreter.hh"
#include "AEFrame.hh"
#include "AEEvalInterpreter.hh"
#include "AEEvalFrame.hh"
#include "AEPrintInterpreter.hh"
#include "AEPrintFrame.hh"

static void InitAE(int argc, const char* argv[]) {
  // we only have to initialize once, so we have to remember
  static bool initialized = false;
  // and initialize, if we do not have yet
  if (!initialized) {
    // all we have to do is initialize our language layer
    AELanguageLayer::Init(argc, argv);
    // and of course remember that we did it!
    initialized = true;
  }
}

void Start(int argc, const char* argv[]) {
  // init ourself, checking required arguments
  InitAE(argc, argv);
  // create a initial thread, needs no arguments
  Thread* thread = Scheduler::NewThread(0, Store::IntToWord(0));
  // create e AEPrintFrame on top of this thread, initialized with the
  // AEPrintInterpreter. This frame gets the "result" of the next one
  // (see below), prints it and exits.
  AEPrintFrame::New(thread, AEPrintInterpreter::self);
  // create e AEEvalFrame on top of this thread, initialized with the
  // AEEvalInterpreter. This frame gets the "result" of the next one
  // (see below) and evaluates this result (the abstract syntax tree.
  AEEvalFrame::New(thread, AEEvalInterpreter::self);
  // create e AEFrame on top of this thread, initialized with the AEInterpreter
  // and the arithmetic expression from the command line
  // (which was stored by the LL on init)
  AEFrame::New(thread, AEInterpreter::self, AELanguageLayer::GetCmdAE());
  // that's it
}

Worker::Result Load(String* ) {
  // don't know if we need this code, but of corse the symbol

  //--** this can be called multiple times, concurrently
  const char* argv[] = {""};
  InitAE(1, argv);

  return Worker::CONTINUE;
}
