//
// Authors:
//   Patrick Cernko <errror@ps.uni-sb.de>
//
// Copyright:
//   Patrick Cernko, 2004
//
// Last Change:
//   $Date: 2004/01/21 15:40:14 $ by $Author: errror $
//   $Revision: 1.1 $
//

#ifndef __AE__AE_PRINT_INTERPRETER_HH__
#define __AE__AE_PRINT_INTERPRETER_HH__

#if defined(INTERFACE)
#pragma interface "AEPrintInterpreter.hh"
#endif

#include "Base.hh"

/**
 * \class AEInterpreter
 *
 * A interpreting Printer for arithmetic expression result.
 */
class AEPrintInterpreter
  : public Interpreter {

private:
  /**
   * the default constructor,
   * empty. Only calls the superclass C'tor.
   */
  AEPrintInterpreter()
    : Interpreter() {
  }

public:
  static AEPrintInterpreter* self; ///! a pointer to the one and only
                                   ///  (singleton) instance of this
                                   ///  interpreter

  /**
   * the static "C'tor" which creates the interpreter and stores a
   * pointer in AEInterpreter::self.
   */
  static void Init();

  /**
   * get the size of a StackFrame
   * \param sFrame the frame whose size should be computed (must be a AEPrintFrame!)
   * \returns the size of the Frame (only if it is a AEPrintFrame!
   */
  virtual u_int GetFrameSize(StackFrame* sFrame);

  /**
   * starts interpreting using the given frame.
   * \param sFrame the frame which should be used for interpretation
   * \returns ???
   */
  virtual Result Run(StackFrame* sFrame);

  /**
   * dummy implementation of pure virtual superclass method.
   */
  virtual u_int GetInArity(ConcreteCode* concreteCode);
  /**
   * dummy implementation of pure virtual superclass method.
   */
  virtual u_int GetOutArity(ConcreteCode* concreteCode);

  /**
   * don't know why I need this!
   */
  virtual void PushCall(Closure* closure);

  /**
   * \returns the name of this interprester as a c-string.
   */
  virtual const char* Identify();

  /**
   * dumps the given StackFrame to stderr.
   * \param sFrame the frame to be dumped.
   */
  virtual void DumpFrame(StackFrame* sFrame);
};


/**
 * \class AEPrintClosure
 * Don't know why I need this yet!
 */
class AEPrintClosure
  : public Closure {
public:
  enum { RECORD_POS, LABEL_POS, SIZE };

  static AEPrintClosure* New(word record, UniqueString* label) {
    ConcreteCode* concreteCode =
      ConcreteCode::New(AEPrintInterpreter::self, 0);
    Closure* closure = Closure::New(concreteCode->ToWord(), SIZE);
    closure->Init(RECORD_POS, record);
    closure->Init(LABEL_POS, label->ToWord());
    return STATIC_CAST(AEPrintClosure *, closure);
  }

  word GetRecord() {
    return Sub(RECORD_POS);
  }
  UniqueString* GetLabel() {
    return UniqueString::FromWordDirect(Sub(LABEL_POS));
  }
};

#endif
